/*
 *  cdc.h
 *
 *  Copyright 2014 Michael Zillgith
 *
 *  This file is part of libIEC61850.
 *
 *  libIEC61850 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libIEC61850 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with libIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  See COPYING file for the complete license text.
 */

#ifndef CDC_H_
#define CDC_H_

#define CDC_OPTION_PICS_SUBST 1
#define CDC_OPTION_BLK_ENA 2
#define CDC_OPTION_DESC 4
#define CDC_OPTION_DESC_UNICODE 8

#define CDC_OPTION_AC_DLNDA 16
#define CDC_OPTION_AC_DLN 32

#define CDC_OPTION_UNIT 64

#define CDC_OPTION_FROZEN_VALUE 128

#define CDC_OPTION_ADDR 256
#define CDC_OPTION_ADDINFO 512

#define CDC_OPTION_INST_MAG 1024
#define CDC_OPTION_RANGE 2048

#define CDC_OPTION_UNIT_MULTIPLIER 4096

#define CDC_OPTION_AC_SCAV 8192

#define CDC_OPTION_MIN 16384
#define CDC_OPTION_MAX 32768

#define CDC_OPTION_AC_CLC_O 65536

#define CDC_OPTION_RANGE_ANG 131072


/****************************************************
 * Constructed Attribute Classes (CAC)
 ***************************************************/

DataAttribute*
CAC_AnalogueValue_create(char* name, ModelNode* parent, FunctionalConstraint fc, uint8_t triggerOptions,
        bool isIntegerNotFloat);

/**
 * CDC_OPTION_AC_CLC_O
 */
DataAttribute*
CAC_Vector_create(char* name, ModelNode* parent, uint32_t options, FunctionalConstraint fc, uint8_t triggerOptions);

DataAttribute*
CAC_Point_create(char* name, ModelNode* parent, FunctionalConstraint fc, uint8_t triggerOptions, bool hasZVal);

DataAttribute*
CAC_ScaledValueConfig_create(char* name, ModelNode* parent);

DataAttribute*
CAC_Unit_create(char* name, ModelNode* parent, bool hasMagnitude);

/****************************************************
 * Common Data Classes (CDC)
 ***************************************************/

DataObject*
CDC_SPS_create(char* dataObjectName, ModelNode* parent, uint32_t options);

DataObject*
CDC_DPS_create(char* dataObjectName, ModelNode* parent, uint32_t options);

DataObject*
CDC_INS_create(char* dataObjectName, ModelNode* parent, uint32_t options);

DataObject*
CDC_ENS_create(char* dataObjectName, ModelNode* parent, uint32_t options);

DataObject*
CDC_BCR_create(char* dataObjectName, ModelNode* parent, uint32_t options);



/**
 * \brief create a new SEC (Security violation) CDC instance (data object)
 *
 * Allowed parent types are LogicalNode and DataObject.
 *
 * options:
 *   standard (include standard optional elements like extension namespaces and descriptions (d, dU).
 *
 *   CDC_OPTION_ADDR (address of the client causing the security violation)
 *   CDC_OPTION_ADDINFO (additional info text)
 *
 * \param dataObjectName the name of the new object
 * \param parent the parent of the new data object (either a LogicalNode or another DataObject)
 * \param options bit mask to encode required optional elements
 */
DataObject*
CDC_SEC_create(char* dataObjectName, ModelNode* parent, uint32_t options);

/**
 * \brief create a new MV (Measured value) CDC instance (data object)
 *
 * Allowed parent types are LogicalNode and DataObject.
 *
 * possible options:
 *   CDC_OPTION_INST_MAG
 *   CDC_OPTION_RANGE
 *   CDC_OPTION_PICS_SUBST
 *   standard (include standard optional elements like extension namespaces and descriptions (d, dU).
 *
 * \param dataObjectName the name of the new object
 * \param parent the parent of the new data object (either a LogicalNode or another DataObject)
 * \param options bit mask to encode required optional elements
 * \param isIntegerNotFloat if true the AnalogueValue instance have integer instead of float
 *
 */
DataObject*
CDC_MV_create(char* dataObjectName, ModelNode* parent, uint32_t options, bool isIntegerNotFloat);

/**
 * \brief create a new SAV (Sampled analog value) CDC instance (data object)
 *
 * Allowed parent types are LogicalNode and DataObject.
 *
 * possible options:
 *   CDC_OPTION_UNIT
 *   CDC_OPTION_AC_SCAV
 *   CDC_OPTION_MIN
 *   CDC_OPTION_MAX
 *   standard (include standard optional elements like extension namespaces and descriptions (d, dU).
 *
 * \param dataObjectName the name of the new object
 * \param parent the parent of the new data object (either a LogicalNode or another DataObject)
 * \param options bit mask to encode required optional elements
 * \param isIntegerNotFloat if true the AnalogueValue instance have integer instead of float
 *
 */
DataObject*
CDC_SAV_create(char* dataObjectName, ModelNode* parent, uint32_t options, bool isIntegerNotFloat);


#endif /* CDC_H_ */
